import sys
import os
import logging
from typing import Optional

from PySide6.QtWidgets import QMessageBox, QApplication

from utils.config import UserConfig
from utils.database import DatabaseManager
from utils.errors.database.database_config_error import DatabaseConfigError
from utils.errors.database.database_connection_error import DatabaseConnectionError
from utils.setup_logger import setup_logger

from ui.window import LibraryWindow


class LibraryApp():
    def __init__(self, user_config: Optional[UserConfig] = None):
        setup_logger()
        self.logger = logging.getLogger(__name__)
        self.logger.info("Starting")

        self.qt_app = QApplication([])
        self.user_config = user_config

        try:
            self.database_manger = DatabaseManager()
        except DatabaseConfigError as e:
            self.exit_with_error(f"Invalid config: {e.config_name}", e.message)
        except DatabaseConnectionError as e:
            self.exit_with_error(f"Could not connect to database: {e}")
        except FileNotFoundError:
            self.exit_with_error("Configuration not found")
        self.window = LibraryWindow()

    def exit_with_error(self, error: str, additional_text: str = ""):
        self.show_error(error, additional_text)
        self.qt_app.quit()
        sys.exit(1)

    def run(self) -> int:
        self.window.show()
        status = self.qt_app.exec()
        self.cleanup()
        self.logger.info("Exiting")
        return status

    def show_error(self, text: str, detail_text: str = ""):
        error_dialog = QMessageBox()
        error_dialog.setIcon(QMessageBox.Icon.Critical)
        error_dialog.setWindowTitle("Error")
        error_dialog.setText(text)
        if detail_text:
            error_dialog.setInformativeText(detail_text)
        error_dialog.setStandardButtons(QMessageBox.StandardButton.Ok)
        error_dialog.exec()

    def cleanup(self) -> None:
        self.logger.info("Cleaning up")
        self.qt_app.quit()
        self.database_manger.cleanup()


if __name__ == "__main__":
    library_app = LibraryApp()
    sys.exit(library_app.run())
